/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/

#include "service_icecast.h"
#include "networkaccess.h"
#include "utilities.h"
#include "debug.h"

#include <QtCore>

/*
********************************************************************************
*                                                                              *
*    Class IceCast                                                             *
*                                                                              *
********************************************************************************
*/
IceCast::IceCast() : Service("Icecast", SERVICE::ICECAST, false)
{
    Debug::debug() << "IceCast -> create";
   
    /* restore state  */
    set_state(SERVICE::NO_DATA);
}

void IceCast::reload()
{
    /* register update */    
    set_state(SERVICE::NO_DATA);    
    emit stateChanged();
}

void IceCast::load()
{
    Debug::debug() << "IceCast -> load";
    if(state() == SERVICE::DOWNLOADING)
      return;
    
    set_state(SERVICE::DOWNLOADING);
    
    emit stateChanged();

    slot_send_request();
}


QList<MEDIA::TrackPtr> IceCast::streams()
{
    return m_streams;
}

//! ------------------  slot_send_request --------------------------------------
void IceCast::slot_send_request()
{
    Debug::debug() << "IceCast -> slot_send_request";

    QUrl url("http://dir.xiph.org/yp.xml");

    QObject *reply = HTTP()->get(url);
    connect(reply, SIGNAL(data(QByteArray)), this, SLOT(slot_parse_response(QByteArray)));
    connect(reply, SIGNAL(error(QNetworkReply*)), this, SLOT(slot_error()));
}

//! ------------------  slot_parse_response ------------------------------------
void IceCast::slot_parse_response(QByteArray bytes)
{

    if (bytes.isEmpty()) {
        set_state(SERVICE::ERROR);
        emit stateChanged();
        return;
    }
    
    /* parse data */
    QXmlStreamReader xml(bytes);

    MEDIA::TrackPtr mi = MEDIA::TrackPtr(0);

    while(!xml.atEnd() && !xml.hasError()) {
      xml.readNext();
      if (xml.isStartElement()) {
          if(xml.name() == "entry") {
              //! new stream entry
              mi = MEDIA::TrackPtr(new MEDIA::Track());
              mi->setType(TYPE_STREAM);
          } else if (xml.name() == "server_name") {
              mi->name = QString(xml.readElementText());
          } else if (xml.name() == "listen_url") {
              mi->url  = QString(xml.readElementText());

              // 0.0.57 les noms de stations apparaissent en "Unspecified name"
              QString server_name = mi->name;
              if(server_name == QString("Unspecified name") || server_name.isEmpty() )
              {
                  mi->name = mi->url;
              }
          } else if (xml.name() == "genre") {
              mi->categorie = QString(xml.readElementText());
          }
      }
      else if (xml.isEndElement()) {
          if(xml.name() == "entry") {
              //! if stream OK --> add it
              if (!mi->url.isEmpty() && !mi->name.isEmpty())
                m_streams.append(mi);
          }
      }
    }

    qSort(m_streams.begin(), m_streams.end(), MEDIA::compareStreamCategorie);
    
    
    /* emit change */    
    set_state(SERVICE::DATA_OK);
    emit stateChanged();
}


void IceCast::slot_error()
{
      set_state(SERVICE::ERROR);
      emit stateChanged();
}
