/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/
#ifndef _NETWORKACCESS_H_
#define _NETWORKACCESS_H_

#include <QtNetwork>
#include <QHash>
#include <QThread>

#include "constants.h"

/*
********************************************************************************
*                                                                              *
*    Class NetworkReply                                                        *
*                                                                              *
********************************************************************************
*/
class NetworkReply : public QObject
{
Q_OBJECT

public:
    NetworkReply(QNetworkReply* networkReply);

public slots:
    void finished();
    void requestError(QNetworkReply::NetworkError);
    void downloadProgress(qint64 bytesReceived, qint64 bytesTotal);
    void readTimeout();
    
signals:
    void data(QByteArray);
    void error(QNetworkReply*);
    void finished(QNetworkReply*);

private:
    void setupReply();

private:
    QNetworkReply    *m_networkReply;
    QTimer           *m_timer;
    int               m_retryCount;
};

/*
********************************************************************************
*                                                                              *
*    Class NetworkAccess                                                       *
*                                                                              *
********************************************************************************
*/
class NetworkAccess : public QObject
{
Q_OBJECT

public:
    NetworkAccess(QObject* parent = 0);
    QNetworkReply* request(QUrl url,int operation = QNetworkAccessManager::GetOperation, QByteArray data = QByteArray());
    NetworkReply* get(QUrl url);
    NetworkReply* head(QUrl url);
    NetworkReply* post(QUrl url, QByteArray data = QByteArray());

private:
    QNetworkRequest buildRequest(QUrl url);
};

/*
********************************************************************************
*                                                                              *
*    Class NetworkManager                                                      *
*                                                                              *
********************************************************************************
*/
#define HTTP() (NetworkManager::instance()->http())

class NetworkManager
{
  static NetworkManager         *INSTANCE;

public:
  NetworkManager();

  static NetworkManager* instance() { return INSTANCE; }

  NetworkAccess* http();

  QNetworkAccessManager* accessManager() {return m_network_access_manager;}

private :
  QHash<QThread*, NetworkAccess *> m_http;
  QNetworkAccessManager           *m_network_access_manager;
};



typedef QPointer<QObject> ObjectPointer;
Q_DECLARE_METATYPE(ObjectPointer)

#endif // _NETWORKACCESS_H_
